@@ -1,9 +1,15 @@
 package Locale::CLDR::It::Any;
-# This file auto generated
-#	on Sun 23 Mar  7:50:38 pm GMT
 
-use v5.18;
+# This file auto generated
+#	on Sun  1 Jun  3:22:38 pm GMT
+
+use version;
+
+our $VERSION = version->declare('v0.25.2');
+
+use v5.10;
 use mro 'c3';
+use if $^V ge v5.12.0, feature => 'unicode_strings';
 
 use Moose;
 
