@@ -1,9 +1,15 @@
 package Locale::CLDR::Mg::Any;
-# This file auto generated
-#	on Sun 23 Mar  8:31:47 pm GMT
 
-use v5.18;
+# This file auto generated
+#	on Sun  1 Jun  3:47:32 pm GMT
+
+use version;
+
+our $VERSION = version->declare('v0.25.2');
+
+use v5.10;
 use mro 'c3';
+use if $^V ge v5.12.0, feature => 'unicode_strings';
 
 use Moose;
 
