@@ -1,39 +1,92 @@
-package Locale::CLDR::NumberFormatter v0.25.0;
+package Locale::CLDR::NumberFormatter;
 
-use v5.18;
+use version;
+
+our $VERSION = version->declare('v0.25.2');
+
+
+use v5.10;
 use mro 'c3';
 use utf8;
+use if $^V ge v5.12.0, feature => 'unicode_strings';
 
 use Moose::Role;
 
 # This method assumes a numeric numbering system. It will have to be re written to handle algorithmic systems later 
 sub format_number {
-	my ($self, $number, $format, $currency) = @_;
+	my ($self, $number, $format, $currency, $for_cash) = @_;
 	
 	$format //= '0';
 	
+	my $currency_data;
+	
 	# Check if we need a currency and have not been given one.
 	# In that case we look up the default currency for the locale
-	if ($format =~ tr/¤/¤/ && ! defined $currency) {
-		my $default_currency_bundle = $self->_find_bundle('default_currency');
-		$currency = $default_currency_bundle->default_currency()->{$self->territory};
+	if ($format =~ tr/¤/¤/) {
+	
+		$for_cash //=0;
+		
+		$currency = $self->default_currency()
+			if ! defined $currency;
+		
+		$currency_data = $self->_get_currency_data($currency);
+		
+		$currency = $self->currency_symbol($currency);
 	}
 	
-	$format = $self->parse_number_format($format);
+	$format = $self->parse_number_format($format, $currency, $currency_data, $for_cash);
 	
-	$number = $self->get_formatted_number($number, $format);
+	$number = $self->get_formatted_number($number, $format, $currency_data, $for_cash);
 	
 	return $number;
 }
 
+sub add_currency_symbol {
+	my ($self, $format, $symbol) = @_;
+	
+	$format =~ s/¤/$symbol/;
+	
+	return $format;
+}
+
+sub _get_currency_data {
+	my ($self, $currency) = @_;
+	
+	my $currency_data = $self->currency_fractions($currency);
+	
+	return $currency_data;
+}
+
+sub _get_currency_rounding {
+
+	my ($self, $currency_data, $for_cash) = @_;
+	
+	my $rounder = $for_cash ? 'cashrounding' : 'rounding' ;
+	
+	return $currency_data->{$rounder};
+}
+
+sub _get_currency_digits {
+	my ($self, $currency_data, $for_cash) = @_;
+	
+	my $digits = $for_cash ? 'cashdigits' : 'digits' ;
+	
+	return $currency_data->{$digits};
+}
+
+
 sub parse_number_format {
-	my ($self, $format) = @_;
+	my ($self, $format, $currency, $currency_data, $for_cash) = @_;
 
 	use feature 'state';
 	
 	state %cache;
+	
 	return $cache{$format} if exists $cache{$format};
 	
+	$format = $self->add_currency_symbol($format, $currency)
+		if defined $currency;
+	
 	my ($positive, $negative) = $format =~ /^((?:'[^']*')++ | [^';]+)+ (?:;(.+))?$/x;
 	
 	my $type = 'positive';
@@ -85,6 +138,9 @@ sub parse_number_format {
 		my $rounding = $to_parse =~ /([1-9][0-9]*(?:\.[0-9]+)?)/;
 		$rounding ||= 0;
 		
+		$rounding = $self->_get_currency_rounding($currency_data, $for_cash)
+			if defined $currency;
+		
 		my ($integer, $decimal) = split /\./, $to_parse;
 		
 		my ($minimum_significant_digits, $maximum_significant_digits, $minimum_digits);
@@ -140,13 +196,36 @@ sub parse_number_format {
 	return $cache{$format};
 }
 
-# Crappy rounding function
+# Rounding function
 sub round {
-	return int ($_[1] + .5 );
+	my ($self, $number, $increment, $decimal_digits) = @_;
+
+	if ($increment ) {
+		$number /= $increment;
+		$number = int ($number + .5 );
+		$number *= $increment;
+	}
+	
+	if ( $decimal_digits ) {
+		$number *= 10 ** $decimal_digits;
+		$number = int $number;
+		$number /= 10 ** $decimal_digits;
+		
+		my ($decimal) = $number =~ /(\..*)/; 
+		$decimal //= '.'; # No fraction so add a decimal point
+		
+		$number = int ($number) . $decimal . ('0' x ( $decimal_digits - length( $decimal ) +1 ));
+	}
+	else {
+		# No decimal digits wanted
+		$number = int $number;
+	}
+	
+	return $number;
 }
 
 sub get_formatted_number {
-	my ($self, $number, $format) = @_;
+	my ($self, $number, $format, $currency_data, $for_cash) = @_;
 	
 	my @digits = $self->get_digits;
 	my @number_symbols_bundles = reverse $self->_find_bundle('number_symbols');
@@ -159,8 +238,14 @@ sub get_formatted_number {
 	
 	$number *= $format->{$type}{multiplier};
 	
-	if ($format->{rounding}) {
-		$number = $format->{$type}{rounding} * $self->round( ($number / $format->{$type}{rounding}) );
+	if ($format->{rounding} || defined $for_cash) {
+		my $decimal_digits = 0;
+		
+		if (defined $for_cash) {
+			$decimal_digits = $self->_get_currency_digits($currency_data, $for_cash)
+		}
+		
+		$number = $self->round($number, $format->{$type}{rounding}, $decimal_digits);
 	}
 	
 	my $pad_zero = $format->{$type}{minimum_digits} - length "$number";
@@ -178,7 +263,7 @@ sub get_formatted_number {
 		@parts = ($minor);
 		while (1) {
 			my ($major) = $integer =~ /($major_group)$/;
-			if (length $major) {
+			if (defined $major && length $major) {
 				$integer =~ s/$major_group$//;
 				unshift @parts, $major;
 			}
